<?php

namespace App\Http\Controllers\User;

use App\Models\Plan;
use App\Models\User;
use App\Constants\Status;
use App\Models\Promotion;
use App\Models\DeviceToken;
use App\Models\Transaction;
use Illuminate\Http\Request;
use App\Models\CreditHistory;
use App\Models\PromotionView;
use Illuminate\Support\Carbon;
use Illuminate\Validation\Rule;
use App\Lib\GoogleAuthenticator;
use App\Models\AdminNotification;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;

class UserController extends Controller
{
    public function home()
    {
        $pageTitle = 'Dashboard';
        $user = Auth::user();

        $totalTransaction = Transaction::where('user_id', $user->id)->count();
        $totalAd = Promotion::where('user_id', $user->id)->count();
        $totalCreditHis = CreditHistory::where('user_id', $user->id)->count();
        $plan = Plan::find($user->plan_id);
        $downLine = User::where('ref_by', $user->id)->count();
        $logs = Transaction::where('user_id', $user->id)->latest()->limit(10)->get();
        $upline = User::where('id', $user->ref_by)->first();

        return view('Template::user.dashboard', compact('pageTitle', 'user', 'totalTransaction', 'totalAd', 'totalCreditHis', 'plan', 'downLine', 'logs', 'upline'));
    }

    public function depositHistory(Request $request)
    {
        $pageTitle = 'Deposit History';
        $deposits = auth()->user()->deposits()->searchable(['trx'])->with(['gateway'])->orderBy('id', 'desc')->paginate(getPaginate());
        return view('Template::user.deposit_history', compact('pageTitle', 'deposits'));
    }

    public function show2faForm()
    {
        $ga = new GoogleAuthenticator();
        $user = auth()->user();
        $secret = $ga->createSecret();
        $qrCodeUrl = $ga->getQRCodeGoogleUrl($user->username . '@' . gs('site_name'), $secret);
        $pageTitle = '2FA Security';
        return view('Template::user.twofactor', compact('pageTitle', 'secret', 'qrCodeUrl'));
    }

    public function create2fa(Request $request)
    {
        $user = auth()->user();
        $request->validate([
            'key' => 'required',
            'code' => 'required',
        ]);
        $response = verifyG2fa($user, $request->code, $request->key);
        if ($response) {
            $user->tsc = $request->key;
            $user->ts = Status::ENABLE;
            $user->save();
            $notify[] = ['success', 'Two factor authenticator activated successfully'];
            return back()->withNotify($notify);
        } else {
            $notify[] = ['error', 'Wrong verification code'];
            return back()->withNotify($notify);
        }
    }

    public function disable2fa(Request $request)
    {
        $request->validate([
            'code' => 'required',
        ]);

        $user = auth()->user();
        $response = verifyG2fa($user, $request->code);
        if ($response) {
            $user->tsc = null;
            $user->ts = Status::DISABLE;
            $user->save();
            $notify[] = ['success', 'Two factor authenticator deactivated successfully'];
        } else {
            $notify[] = ['error', 'Wrong verification code'];
        }
        return back()->withNotify($notify);
    }

    public function transactions()
    {
        $pageTitle = 'Transactions';
        $remarks = Transaction::where('user_id', auth()->id())->distinct('remark')->orderBy('remark')->get('remark', 'id');

        $transactions = Transaction::where('user_id', auth()->id())->searchable(['trx'])->filter(['trx_type', 'remark'])->orderBy('id', 'desc')->paginate(getPaginate());

        return view('Template::user.transactions', compact('pageTitle', 'transactions', 'remarks'));
    }

    public function userData()
    {
        $user = auth()->user();

        if ($user->profile_complete == Status::YES) {
            return to_route('user.home');
        }

        $pageTitle  = 'User Data';
        $info       = json_decode(json_encode(getIpInfo()), true);
        $mobileCode = @implode(',', $info['code']);
        $countries  = json_decode(file_get_contents(resource_path('views/partials/country.json')));

        return view('Template::user.user_data', compact('pageTitle', 'user', 'countries', 'mobileCode'));
    }

    public function userDataSubmit(Request $request)
    {

        $user = auth()->user();

        if ($user->profile_complete == Status::YES) {
            return to_route('user.home');
        }

        $countryData  = (array)json_decode(file_get_contents(resource_path('views/partials/country.json')));
        $countryCodes = implode(',', array_keys($countryData));
        $mobileCodes  = implode(',', array_column($countryData, 'dial_code'));
        $countries    = implode(',', array_column($countryData, 'country'));

        $request->validate([
            'country_code' => 'required|in:' . $countryCodes,
            'country'      => 'required|in:' . $countries,
            'mobile_code'  => 'required|in:' . $mobileCodes,
            'username'     => 'required|unique:users|min:6',
            'mobile'       => ['required', 'regex:/^([0-9]*)$/', Rule::unique('users')->where('dial_code', $request->mobile_code)],
        ]);


        if (preg_match("/[^a-z0-9_]/", trim($request->username))) {
            $notify[] = ['info', 'Username can contain only small letters, numbers and underscore.'];
            $notify[] = ['error', 'No special character, space or capital letters in username.'];
            return back()->withNotify($notify)->withInput($request->all());
        }

        $user->country_code = $request->country_code;
        $user->mobile       = $request->mobile;
        $user->username     = $request->username;


        $user->address = $request->address;
        $user->city = $request->city;
        $user->state = $request->state;
        $user->zip = $request->zip;
        $user->country_name = @$request->country;
        $user->dial_code = $request->mobile_code;

        $user->profile_complete = Status::YES;
        $user->save();

        return to_route('user.home');
    }


    public function addDeviceToken(Request $request)
    {

        $validator = Validator::make($request->all(), [
            'token' => 'required',
        ]);

        if ($validator->fails()) {
            return ['success' => false, 'errors' => $validator->errors()->all()];
        }

        $deviceToken = DeviceToken::where('token', $request->token)->first();

        if ($deviceToken) {
            return ['success' => true, 'message' => 'Already exists'];
        }

        $deviceToken          = new DeviceToken();
        $deviceToken->user_id = auth()->user()->id;
        $deviceToken->token   = $request->token;
        $deviceToken->is_app  = Status::NO;
        $deviceToken->save();

        return ['success' => true, 'message' => 'Token saved successfully'];
    }

    public function downloadAttachment($fileHash)
    {
        $filePath = decrypt($fileHash);
        $extension = pathinfo($filePath, PATHINFO_EXTENSION);
        $title = slug(gs('site_name')) . '- attachments.' . $extension;
        try {
            $mimetype = mime_content_type($filePath);
        } catch (\Exception $e) {
            $notify[] = ['error', 'File does not exists'];
            return back()->withNotify($notify);
        }
        header('Content-Disposition: attachment; filename="' . $title);
        header("Content-Type: " . $mimetype);
        return readfile($filePath);
    }

    public function plans()
    {
        $creditPlan = Plan::where('publication_status', Status::PUBLISHED)->first();

        if (!$creditPlan) {
            $notify[] = ['error', 'Sorry there is no available plan'];
            return back()->withNotify($notify);
        }

        $plans = Plan::where('publication_status', Status::PUBLISHED)->paginate(getPaginate());
        $pageTitle = 'All Plans';
        return view('Template::user.plan.all', compact('pageTitle', 'plans'));
    }

    public function planBuy(Request $request)
    {
        $request->validate([
            'id' => 'required|exists:plans,id'
        ]);

        $creditPlan = Plan::where('publication_status', Status::PUBLISHED)->findOrFail($request->id);
        $user = auth()->user();

        if ($creditPlan->price > $user->balance) {
            $notify[] = ['error', 'Sorry you have not sufficient balance to purchase plan'];
            return to_route('user.deposit.index')->withNotify($notify);
        }

        $creditPlan->increment('sold');
        $creditPlan->save();

        $user->balance -= $creditPlan->price;
        $user->credit += $creditPlan->credit;
        $user->plan_id = $creditPlan->id;
        $user->save();

        $transaction = new Transaction();
        $transaction->user_id = $user->id;
        $transaction->amount = $creditPlan->price;
        $transaction->charge = 0;
        $transaction->post_balance = $user->balance;
        $transaction->trx_type = '-';
        $transaction->remark = 'credit_purchase';
        $transaction->trx = getTrx();
        $transaction->details = 'Purchased ' . $creditPlan->name . ' Plan';
        $transaction->save();

        $creditHistory = new CreditHistory();
        $creditHistory->user_id = $user->id;
        $creditHistory->plan_id = $creditPlan->id;
        $creditHistory->type = '+';
        $creditHistory->price = $creditPlan->price;
        $creditHistory->credit = $creditPlan->credit;
        $creditHistory->credit_post_balance = $user->credit;
        $creditHistory->details = 'Purchased ' . $creditPlan->name . ' plan';
        $creditHistory->save();

        $adminNotification = new AdminNotification();
        $adminNotification->user_id = $user->id;
        $adminNotification->title = $creditHistory->details . ' ' . getAmount($creditPlan->price) . ' ' . $general->cur_text;
        $adminNotification->click_url = route('admin.users.credit.history', $user->id);
        $adminNotification->save();

        notify($user, 'PLAN_PURCHASED', [
            'plan_name'    => $creditPlan->name,
            'credit'       => $creditPlan->credit,
            'plan_price'   => showAmount($creditPlan->price, currencyFormat: false),
            'post_balance' => showAmount($user->balance, currencyFormat: false),
            'post_credit'  => $user->credit,
        ]);

        $notify[] = ['success', 'You have purchased credit plan successfully'];
        return to_route('user.credit.history')->withNotify($notify);
    }

    public function creditHistories()
    {
        $pageTitle = 'Credit History';
        $user = auth()->user();
        $logs = CreditHistory::where('user_id', $user->id)->latest()->paginate(getPaginate());
        return view('Template::user.plan.credit_log', compact('pageTitle', 'logs'));
    }


    // Promotion
    public function promotion()
    {
        $user = Auth::user();

        if (!getAmount($user->credit)) {
            $notify[] = ['error', 'Sorry you have no credit'];
            return to_route('user.plan.all')->withNotify($notify);
        }

        $pageTitle = 'Create a promotion';
        return view('Template::user.promotion.create', compact('pageTitle'));
    }

    public function promotionConfirm(Request $request)
    {
        $general = gs();

        $request->validate([
            'url' => 'required|url',
            'click' => 'required|integer|gt:0',
            'credit' => 'required|integer|gte:' . getAmount($general->min_credit),
            'publication_status' => 'required|in:0,1',
            'title' => 'required|max:191',
            'image' => 'required|mimes:png,jpg,jpeg',
        ]);

        $totalRequiredCredit = $request->click * $request->credit;
        $user = Auth::user();

        if ($totalRequiredCredit > $user->credit) {
            $notify[] = ['error', 'Sorry You have not sufficient credit to promote your advertisement'];
            return to_route('user.plan.all')->withNotify($notify);
        }

        $promotion = new Promotion();
        $promotion->status = Status::CREATED;
        $promotion->title = $request->title;
        $promotion->user_id = $user->id;
        $promotion->url = $request->url;
        $promotion->click = $request->click;
        $promotion->credit = $request->credit;
        $promotion->total_credit = $totalRequiredCredit;
        $promotion->publication_status = $request->publication_status;

        if ($request->hasFile('image')) {
            try {
                $promotion->image = fileUploader($request->image, getFilePath('advertisement'), getFileSize('advertisement'));
            } catch (\Exception $exp) {
                $notify[] = ['errors', 'Image could not be uploaded.'];
                return back()->withNotify($notify);
            }
        }

        $promotion->save();

        $user->credit -= $totalRequiredCredit;
        $user->save();

        $creditHistory = new CreditHistory();
        $creditHistory->user_id = $user->id;
        $creditHistory->type = '-';
        $creditHistory->credit = $totalRequiredCredit;
        $creditHistory->credit_post_balance = $user->credit;
        $creditHistory->details = 'Less credit for promotion';
        $creditHistory->save();

        $adminNotification = new AdminNotification();
        $adminNotification->user_id = $user->id;
        $adminNotification->title = 'Create promotion';
        $adminNotification->click_url = route('admin.ad.users');
        $adminNotification->save();

        notify($user, 'PROMOTION_CREATE_COMPLETE', [
            'title' => $promotion->title,
            'url' => $promotion->url,
            'post_credit' => getAmount($user->credit)
        ]);

        $notify[] = ['success', 'Your promotion added successfully'];
        return to_route('user.promotion.my')->withNotify($notify);
    }

    public function myPromotion()
    {
        $pageTitle = 'My promotions';
        $user = Auth::user();
        $logs = Promotion::where('user_id', $user->id)->latest()->paginate(getPaginate());
        return view('Template::user.promotion.logs', compact('pageTitle', 'logs'));
    }

    public function promotionEdit(Request $request)
    {

        $request->validate([
            'id' => 'required|exists:promotions,id',
            'url' => 'required|url',
            'publication_status' => 'required|in:0,1',
            'title' => 'required|max:191',
        ]);

        $findPromotion = Promotion::where('user_id', Auth::user()->id)
            ->where('id', $request->id)
            ->where('status', '!=', Status::REJECTED)
            ->firstOrFail();

        $findPromotion->url = $request->url;
        $findPromotion->title = $request->title;
        $findPromotion->publication_status = $request->publication_status;
        $findPromotion->status = Status::EDITED;

        if ($request->hasFile('image')) {
            try {
                $findPromotion->image = fileUploader($request->image, getFilePath('advertisement'), getFileSize('advertisement'), $findPromotion->image);
            } catch (\Exception $exp) {
                $notify[] = ['errors', 'Image could not be uploaded.'];
                return back()->withNotify($notify);
            }
        }

        $findPromotion->save();

        $notify[] = ['success', 'Your promotion info edited successfully'];
        return back()->withNotify($notify);
    }

    public function promotionDelete(Request $request)
    {
        $request->validate([
            'id' => 'required|exists:promotions,id',
        ]);

        $findPromotion = Promotion::where('user_id', Auth::user()->id)
            ->where('id', $request->id)
            ->firstOrFail();

        fileManager()->removeFile(getFilePath('advertisement') . '/' . $findPromotion->image);

        $findPromotion->delete();

        $notify[] = ['success', 'Your promotion deleted successfully'];
        return back()->withNotify($notify);
    }

    // downlines
    public function downlines()
    {
        $pageTitle = 'Downlines';
        $user = Auth::user();
        $users = User::where('ref_by', $user->id)->latest()->paginate(getPaginate());
        return view('Template::user.downlines', compact('pageTitle', 'users'));
    }

    // advertisements
    public function viewAdvertisement($id)
    {
        $general = gs();
        if (session()->get('duration') > now()) {
            $time = session()->get('duration');
            $differentTime = $time->diffForHumans(now(), true);
            $notify[] = ['error', 'You will be able to see ads after ' . $differentTime];
            return back()->withNotify($notify);
        }

        $user = Auth::user();
        $originId = decrypt($id);
        $advertisement = Promotion::findOrFail($originId);

        $view = PromotionView::where('promotion_id', $originId)
            ->where('user_id', $user->id)
            ->orderBy('id', 'DESC')
            ->first();

        if ($view) {
            $nextTime = $view->created_at->addHours($general->watch_limit);
            if ($nextTime >= now()) {
                $differentTime = $nextTime->diffForHumans(now(), true);
                $notify[] = ['error', 'You will be able to see this ad after ' . $differentTime];
                return back()->withNotify($notify);
            }
        }

        $user->credit += $advertisement->credit;
        $user->save();

        $advertisement->clicked += 1;
        $advertisement->status = $advertisement->clicked >= $advertisement->click ? 2 : 1;
        $advertisement->save();

        $creditHistory = new CreditHistory();
        $creditHistory->user_id = $user->id;
        $creditHistory->credit = $advertisement->credit;
        $creditHistory->type = '+';
        $creditHistory->credit_post_balance = $user->credit;
        $creditHistory->details = 'Added credit for view advertisement';
        $creditHistory->save();

        $viewHistory = new PromotionView();
        $viewHistory->user_id = $user->id;
        $viewHistory->promotion_id = $advertisement->id;
        $viewHistory->credit = $advertisement->credit;
        $viewHistory->save();

        if (session()->get('duration')) {
            session()->forget('duration');
        }
        session()->put('duration', Carbon::now()->addSecond($general->duration));

        return redirect($advertisement->url);
    }

    public function advertisements()
    {
        $pageTitle = 'Advertisements';
        $user = Auth::user();

        $ads = Promotion::where('publication_status', Status::PUBLISHED)
            ->where('status', Status::RUNNING)
            ->where(function ($query) use ($user) {
                $query->orWhere('user_id', '!=', $user->id)->orWhere('user_id', null);
            })
            ->orderBy('credit', 'DESC')
            ->paginate(getPaginate());

        return view('Template::user.advertisements', compact('pageTitle', 'user', 'ads'));
    }
}
